﻿using C1.Chart;
using C1.Win.Chart;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace FlexChartSample
{
    public partial class Form1 : Form
    {
        PointF _clickedPoint;

        public Form1()
        {
            InitializeComponent();

            SetupChart();

            //save the clicked point on MouseDown and refresh the Chart to draw selection point
            flexChart1.MouseDown += (s, e) => { if (e.Button == MouseButtons.Left) _clickedPoint = e.Location; Refresh(); };

            flexChart1.Rendered += FlexChart1_Rendered;
        }

        private void FlexChart1_Rendered(object sender, RenderEventArgs e)
        {
            if (_clickedPoint == null || _clickedPoint.IsEmpty) return;
            
            //draw an ellipse on Chart to show selected point
            e.Engine.SetFill(Brushes.Red);
            e.Engine.SetStroke(Brushes.Red);
            e.Engine.SetStrokeThickness(1);
            e.Engine.DrawEllipse(_clickedPoint.X, _clickedPoint.Y, 3, 3);
        }

        private void AddPointToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (_clickedPoint == null || _clickedPoint.IsEmpty) return;

            var DataSeries = flexChart1.DataSource as List<PointF>;
            var closestIndex = FindClosestIndex(DataSeries, _clickedPoint);
            var newDataPoint = flexChart1.PointToData(_clickedPoint);

            if (DataSeries[closestIndex].X < newDataPoint.X)
                closestIndex++;

            //add new point to data
            DataSeries.Insert(closestIndex, newDataPoint);

            //rebind flexchart to draw new point
            flexChart1.Rebind();

            //clear the selection point
            _clickedPoint = Point.Empty;
        }

        /// <summary>
        /// Finds the index of the point in the list which is closest to the specified point
        /// </summary>
        /// <param name="dataPoints">List of Data Points being used by the FlexChart</param>
        /// <param name="p">point for which you need to find the closest point from the List</param>
        /// <returns>Index of the closest point in the List</returns>
        public int FindClosestIndex(List<PointF> dataPoints, PointF p)
        {
            double closest = double.MaxValue;
            int closestIndex = -1;

            for (int i = 0; i < dataPoints.Count; i++)
            {
                var coordPoint = flexChart1.DataToPoint(dataPoints[i]);

                double dx = coordPoint.X - p.X;
                double dy = coordPoint.Y - p.Y;
                double distance = dx * dx + dy * dy;

                if (distance >= closest)
                    continue;

                closestIndex = i;
                closest = distance;
            }

            return closestIndex;
        }

        private void clearSelectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(_clickedPoint.IsEmpty) return;

            _clickedPoint = Point.Empty;
            flexChart1.Refresh();
        }

        /// <summary>
        /// Setups Chart binding
        /// </summary>
        private void SetupChart()
        {
            flexChart1.Series.Clear();
            flexChart1.ChartType = ChartType.LineSymbols;
            flexChart1.DataSource = GetSampleData();

            //Creating and adding a series in FlexChart
            var items = new Series
            {
                BindingX = "X",
                Binding = "Y"
            };
            flexChart1.Series.Add(items);
        }

        /// <summary>
        /// Gets Sample data for the FlexChart
        /// </summary>
        public List<PointF> GetSampleData()
        {
            List<PointF> _list = new List<PointF>
            {
                new PointF() {X = 0, Y = 0},
                new PointF() {X = 10, Y = 0.5f},
                new PointF() {X = 20, Y = 0.5f},
                new PointF() {X = 30, Y = 1},
                new PointF() {X = 40, Y = 1},
                new PointF() {X = 50, Y = 0.5f},
                new PointF() {X = 60, Y = 0.5f},
                new PointF() {X = 70, Y = 0},
            };
            return _list;
        }

        private void deletePointAtSelectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (_clickedPoint == null || _clickedPoint.IsEmpty) return;

            var dataSeries = flexChart1.DataSource as List<PointF>;
            int pointToDelete = -1;

            //finding a point close to our click location
            for (int i = 0; i < dataSeries.Count; i++)
            {
                //calculate the distance for each point
                var coordPoint = flexChart1.DataToPoint(dataSeries[i]);
                double dx = coordPoint.X - _clickedPoint.X;
                double dy = coordPoint.Y - _clickedPoint.Y;
                double distance = dx * dx + dy * dy;

                //if distance is less than 20
                if (distance < 20)
                {
                    //mark the index of this point
                    pointToDelete = i;
                    break;
                }
            }

            if (pointToDelete != -1)
            {
                //delete the point from datasource and rebind the Grid
                dataSeries.RemoveAt(pointToDelete);
                _clickedPoint = PointF.Empty;
                flexChart1.Rebind();
            }
        }
    }
}
